import 'dart:io';
import 'dart:async';
import 'package:path/path.dart' as path;
import '../util/utilities.dart';

void createWidget(String widgetName) {
  stdout.write(
      'Do you want to create a common widget or a feature widget? (common/feature): ');
  String choice = stdin.readLineSync()!.toLowerCase();

  if (choice == 'common') {
    createWidgetProcess(widgetName, 'lib/core/ui/widgets', true);
  } else if (choice == 'feature') {
    stdout.write('Enter your feature name: ');
    String featureName = stdin.readLineSync()!.toLowerCase();
    Directory destinationDirectory = Directory('lib/features/$featureName');
    if (destinationDirectory.existsSync()) {
      createWidgetProcess(
          widgetName, 'lib/features/$featureName/presentation/screen', false);
    } else {
      print('Feature $featureName does not exist.');
    }
  } else {
    print('Invalid choice. Please choose "common" or "feature".');
    exit(1);
  }
}

Future<void> createWidgetProcess(
    String widgetName, String destinationPath, bool isCommon) async {
  String destinationFilePath = isCommon
      ? '$destinationPath/$widgetName.dart'
      : '$destinationPath/widgets/$widgetName.dart';
  if (doesFileExist(destinationFilePath)) {
    print(
        'File at $destinationFilePath already exists. Please choose a different widget name');
    exit(1);
  }

  String currentFilePath = Platform.script.toFilePath();
  List<String> pathSegments = currentFilePath.split(Platform.pathSeparator);
  if (pathSegments.length > 2) {
    pathSegments.removeRange(pathSegments.length - 5, pathSegments.length);
  }
  String projectDirectoryPath = pathSegments.join(Platform.pathSeparator);
  String sourceFolderPath =
      '$projectDirectoryPath/lib/create_widget/basic_widget';
  grantPermissions(sourceFolderPath);

  if (!Directory(sourceFolderPath).existsSync()) {
    print('Error: $sourceFolderPath folder does not exist');
    return;
  }

  try {
    destinationPath =
        isCommon ? destinationPath : '$destinationPath/widgets';
    await copyFolder(sourceFolderPath, destinationPath, widgetName);
    processFiles(destinationPath, '/*', '', false);
    processFiles(destinationPath, '*/', '', false);
    renameFilesWithSubstring(destinationPath, 'basic_widget', widgetName);
    processFiles(destinationPath, 'BasicWidget', widgetName, true);
    String? name = getPubspecInfo('name');
    if (name != null) {
      processFiles(destinationPath, 'flutter_clean_architecture', name, false);
    }
    print('Widget $widgetName created at $destinationPath');
  } catch (e) {
    print('Error : $e');
  }
}

Future<void> copyFolder(
    String sourcePath, String destinationPath, String widgetName) async {
  Directory sourceDir = Directory(sourcePath);
  Directory destDir = Directory(destinationPath);
  if (!await destDir.exists()) {
    await destDir.create(recursive: true);
  }
  List<FileSystemEntity> contents = sourceDir.listSync();
  for (var entity in contents) {
    if (entity is File) {
      String newPath = path.join(destinationPath, path.basename(entity.path));
      await entity.copy(newPath);
    }
  }
}
