import 'dart:io';
import 'package:yaml/yaml.dart';

void grantPermissions(String folderPath) {
  Process.runSync('chmod', ['-R', 'u+rX', folderPath]);
}

String toPascalCase(String input) {
  List<String> words = input.split(RegExp(r'[^a-zA-Z0-9]'));
  words.removeWhere((element) => element.isEmpty);
  return words
      .map((word) => word[0].toUpperCase() + word.substring(1).toLowerCase())
      .join('');
}

bool doesFileExist(String filePath) {
  File file = File(filePath);
  return file.existsSync();
}

void renameFilesWithSubstring(String path, String substring, String newName) {
  Directory(path).listSync(recursive: true).forEach((entity) {
    if (entity is File) {
      String fileName =
          entity.path.split(Platform.pathSeparator).last.toLowerCase();
      if (fileName.contains(substring.toLowerCase())) {
        String newPath = entity.path
            .replaceAll(substring.toLowerCase(), newName.toLowerCase());
        entity.renameSync(newPath);
      }
    }
  });
}

void processFiles(
    String path, String substring, String replacement, bool isPascalCase) {
  String pascalCase = toPascalCase(replacement);
  Directory(path).listSync(recursive: true).forEach((entity) {
    if (entity is File) {
      File file = entity;
      String content = file.readAsStringSync();
      content = content.replaceAll(
          substring, isPascalCase ? pascalCase : replacement);
      file.writeAsStringSync(content);
    }
  });
}

String? getPubspecInfo(String fieldName) {
  try {
    String currentDirectory = Directory.current.path;
    String pubspecPath = '$currentDirectory/pubspec.yaml';
    File pubspecFile = File(pubspecPath);
    String contents = pubspecFile.readAsStringSync();
    var yaml = loadYaml(contents);
    return yaml[fieldName];
  } catch (e) {
    print('Error reading pubspec.yaml: $e');
    return null;
  }
}